/** @file modbuscli.c MODBUS/UDP client sample for the MSXE3601
*
* FC16 queries are for setting things, FC3 for fetching values
*
* This code doesn't handle timeouts.
*/

#ifdef WIN32
#include "../../Interface_Library/msxe360x_modbus_clientlib.h"
#else
#include <msxe360x_modbus_clientlib.h>
#endif

#include <modbusclient.h>
#include <commonclient.h>

#include <addidata_console.h>
#include <addidata_time.h>
#include <addidata_network.h>

#define MFMARK(__fn) printf("calling MODBUS function %s\n",__fn)

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputInitSequence (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulChannelMask = 0xFF;				// Eight channels activated
	uint32_t ulNbrOfSequence = 0;				// In continuous mode
	uint32_t ulNbrMaxSequenceToTransfer = 0;	// Not used, must be 0
	double dFrequencySelection = 1000.0;		// Acquisition to 1000.0 Hz
	uint32_t pulGain[8] = {1,1,1,1,1,1,1,1};	// Gain to 1 for each channel
	uint32_t ulICPMask = 0;						// Not activated
	uint32_t ulTriggerMask = 0;					// Trigger disabled
	uint32_t ulTriggerMode = 0;					// Not used, must be 0
	uint32_t ulHardwareTriggerEdge = 1;			// Rising edge (default value, not used in this sample)
	uint32_t ulHardwareTriggerCount = 1;		// Cannot be < 1
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;	// Not used, must be 0
	uint32_t ulDataFormat = 0;					// No TimeStamp info, not sequence counter info, no trigger info, only channels values
	uint32_t ulCouplingSelectionMask = 0;		// AC Coupling mode for each channel
	uint32_t ulSeDiffSelectionMask = 0;			// Single-Ended mode for each channel

	MFMARK("MSXE360X__AnalogInputInitSequence");
	ret = modbus_call_MSXE360X__AnalogInputInitSequence (modbus,
														ulChannelMask,
														ulNbrOfSequence,
														ulNbrMaxSequenceToTransfer,
														dFrequencySelection,
														pulGain,
														ulICPMask,
														ulTriggerMask,
														ulTriggerMode,
														ulHardwareTriggerEdge,
														ulHardwareTriggerCount,
														ulByTriggerNbrOfSeqToAcquire,
														ulDataFormat,
														ulCouplingSelectionMask,
														ulSeDiffSelectionMask,
														&CommandStatus);

	// Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputInitSequence");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputInitSequenceEx (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulChannelMask = 0xFF;				// Eight channels activated
	uint32_t ulNbrOfSequence = 0;				// In continuous mode
	uint32_t ulNbrMaxSequenceToTransfer = 0;	// Not used, must be 0
	double dFrequencySelection = 1000.0;		// Acquisition to 1000.0 Hz
	uint32_t pulGain[8] = {1,1,1,1,1,1,1,1};	// Gain to 1 for each channel
	uint32_t ulICPMask = 0;						// Not activated
	uint32_t ulTriggerMask = 0;					// Trigger disabled
	uint32_t ulTriggerMode = 0;					// Not used, must be 0
	uint32_t ulHardwareTriggerEdge = 1;			// Rising edge (default value, not used in this sample)
	uint32_t ulHardwareTriggerCount = 1;		// Cannot be < 1
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;	// Not used, must be 0
	uint32_t ulDataFormat = 0;					// No TimeStamp info, not sequence counter info, no trigger info, only channels values
	uint32_t ulCouplingSelectionMask = 0;		// AC Coupling mode for each channel
	uint32_t ulSeDiffSelectionMask = 0;			// Single-Ended mode for each channel

	MFMARK("MSXE360X__AnalogInputInitSequenceEx");
	ret = modbus_call_MSXE360X__AnalogInputInitSequenceEx (modbus,
														ulChannelMask,
														ulNbrOfSequence,
														ulNbrMaxSequenceToTransfer,
														dFrequencySelection,
														pulGain,
														ulICPMask,
														ulTriggerMask,
														ulTriggerMode,
														ulHardwareTriggerEdge,
														ulHardwareTriggerCount,
														ulByTriggerNbrOfSeqToAcquire,
														ulDataFormat,
														ulCouplingSelectionMask,
														ulSeDiffSelectionMask,
														&CommandStatus);

	// Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputInitSequenceEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputStartSequence (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputStartSequence");

	ret = modbus_call_MSXE360X__AnalogInputStartSequence(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputStartSequence");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputStartSequenceEx (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputStartSequenceEx");

	ret = modbus_call_MSXE360X__AnalogInputStartSequenceEx(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputStartSequenceEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputInitAndStartSequence (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulChannelMask = 0xFF;				// Eight channels activated
	uint32_t ulNbrOfSequence = 0;				// In continuous mode
	uint32_t ulNbrMaxSequenceToTransfer = 0;	// Not used, must be 0
	double dFrequencySelection = 1000.0;		// Acquisition to 1000.0 Hz
	uint32_t pulGain[8] = {1,1,1,1,1,1,1,1};	// Gain to 1 for each channel
	uint32_t ulICPMask = 0;						// Not activated
	uint32_t ulTriggerMask = 0;					// Trigger disabled
	uint32_t ulTriggerMode = 0;					// Not used, must be 0
	uint32_t ulHardwareTriggerEdge = 1;			// Rising edge (default value, not used in this sample)
	uint32_t ulHardwareTriggerCount = 1;		// Cannot be < 1
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;	// Not used, must be 0
	uint32_t ulDataFormat = 0;					// No TimeStamp info, not sequence counter info, no trigger info, only channels values
	uint32_t ulCouplingSelectionMask = 0;		// AC Coupling mode for each channel
	uint32_t ulSeDiffSelectionMask = 0;			// Single-Ended mode for each channel

	MFMARK("MSXE360X__AnalogInputInitAndStartSequence");
	ret = modbus_call_MSXE360X__AnalogInputInitAndStartSequence (modbus,
														ulChannelMask,
														ulNbrOfSequence,
														ulNbrMaxSequenceToTransfer,
														dFrequencySelection,
														pulGain,
														ulICPMask,
														ulTriggerMask,
														ulTriggerMode,
														ulHardwareTriggerEdge,
														ulHardwareTriggerCount,
														ulByTriggerNbrOfSeqToAcquire,
														ulDataFormat,
														ulCouplingSelectionMask,
														ulSeDiffSelectionMask,
														&CommandStatus);

	// Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputInitAndStartSequence");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputInitAndStartSequenceEx (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	uint32_t ulChannelMask = 0xFF;				// Eight channels activated
	uint32_t ulNbrOfSequence = 0;				// In continuous mode
	uint32_t ulNbrMaxSequenceToTransfer = 0;	// Not used, must be 0
	double dFrequencySelection = 1000.0;		// Acquisition to 1000.0 Hz
	uint32_t pulGain[8] = {1,1,1,1,1,1,1,1};	// Gain to 1 for each channel
	uint32_t ulICPMask = 0;						// Not activated
	uint32_t ulTriggerMask = 0;					// Trigger disabled
	uint32_t ulTriggerMode = 0;					// Not used, must be 0
	uint32_t ulHardwareTriggerEdge = 1;			// Rising edge (default value, not used in this sample)
	uint32_t ulHardwareTriggerCount = 1;		// Cannot be < 1
	uint32_t ulByTriggerNbrOfSeqToAcquire = 0;	// Not used, must be 0
	uint32_t ulDataFormat = 0;					// No TimeStamp info, not sequence counter info, no trigger info, only channels values
	uint32_t ulCouplingSelectionMask = 0;		// AC Coupling mode for each channel
	uint32_t ulSeDiffSelectionMask = 0;			// Single-Ended mode for each channel

	MFMARK("MSXE360X__AnalogInputInitAndStartSequenceEx");
	ret = modbus_call_MSXE360X__AnalogInputInitAndStartSequenceEx (modbus,
														ulChannelMask,
														ulNbrOfSequence,
														ulNbrMaxSequenceToTransfer,
														dFrequencySelection,
														pulGain,
														ulICPMask,
														ulTriggerMask,
														ulTriggerMode,
														ulHardwareTriggerEdge,
														ulHardwareTriggerCount,
														ulByTriggerNbrOfSeqToAcquire,
														ulDataFormat,
														ulCouplingSelectionMask,
														ulSeDiffSelectionMask,
														&CommandStatus);

	// Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputInitAndStartSequenceEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputGetSequenceStatus (struct modbus *modbus) {
	struct MSXE360X__AnalogInputGetSequenceStatus_parameters_t Response;
	int ret = 0;

	memset(&Response, 0, sizeof(Response));

	MFMARK("MSXE360X__AnalogInputGetSequenceStatus");
	ret = modbus_call_MSXE360X__AnalogInputGetSequenceStatus (modbus, &Response);

	return handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE360X__AnalogInputGetSequenceStatus");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputGetSequenceStatusEx (struct modbus *modbus) {
	struct MSXE360X__AnalogInputGetSequenceStatusEx_parameters_t Response;
	int ret = 0;

	memset(&Response, 0, sizeof(Response));

	MFMARK("MSXE360X__AnalogInputGetSequenceStatusEx");
	ret = modbus_call_MSXE360X__AnalogInputGetSequenceStatusEx (modbus, &Response);

	return handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE360X__AnalogInputGetSequenceStatusEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputGetSequenceConfiguration (struct modbus *modbus) {
	struct MSXE360X__AnalogInputGetSequenceConfiguration_parameters_t Response;
	int ret = 0;

	memset(&Response, 0, sizeof(Response));

	MFMARK("MSXE360X__AnalogInputGetSequenceConfiguration");
	ret = modbus_call_MSXE360X__AnalogInputGetSequenceConfiguration(modbus, &Response);

	return handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE360X__AnalogInputGetSequenceConfiguration");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputGetSequenceConfigurationEx (struct modbus *modbus) {
	struct MSXE360X__AnalogInputGetSequenceConfigurationEx_parameters_t Response;
	int ret = 0;

	memset(&Response, 0, sizeof(Response));

	MFMARK("MSXE360X__AnalogInputGetSequenceConfigurationEx");
	ret = modbus_call_MSXE360X__AnalogInputGetSequenceConfigurationEx (modbus, &Response);

	return handle_result_of_FC3_query(modbus, ret, "modbus_call_MSXE360X__AnalogInputGetSequenceConfigurationEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputStopSequence (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputStopSequence");

	ret = modbus_call_MSXE360X__AnalogInputStopSequence(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputStopSequence");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputStopSequenceEx (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputStopSequenceEx");

	ret = modbus_call_MSXE360X__AnalogInputStopSequenceEx(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputStopSequenceEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputReleaseSequence (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputReleaseSequence");

	ret = modbus_call_MSXE360X__AnalogInputReleaseSequence(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputReleaseSequence");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputReleaseSequenceEx (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputReleaseSequenceEx");

	ret = modbus_call_MSXE360X__AnalogInputReleaseSequenceEx(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputReleaseSequenceEx");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputStopAndReleaseSequence (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputStopAndReleaseSequence");

	ret = modbus_call_MSXE360X__AnalogInputStopAndReleaseSequence(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputStopAndReleaseSequence");
}

//---------------------------------------------------------------------------------
int sample_MSXE360X__AnalogInputStopAndReleaseSequenceEx (struct modbus *modbus) {
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	MFMARK("MSXE360X__AnalogInputStopAndReleaseSequenceEx");

	ret = modbus_call_MSXE360X__AnalogInputStopAndReleaseSequenceEx(modbus, Dummy, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE360X__AnalogInputStopAndReleaseSequenceEx");
}

//---------------------------------------------------------------------------------
static void usage(char *name)
{
	printf("%s ADDRESS tcp|udp\n",name);
}

//---------------------------------------------------------------------------------
void print_header(void)
{
        printf ("\n\n");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| MSX-E360x MODBUS Sample                           |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| Press ESC to stop an acquisition                  |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n\n");
}

//---------------------------------------------------------------------------------
int main (int argc, char **argv) {
	struct modbus modbus;
	int contype =  SOCK_DGRAM;

	if (argc != 3) {
		usage(argv[0]);
		exit(1);
	}

	if (0 == strcmp(argv[2],"tcp"))
	{
		contype=SOCK_STREAM;
	}
	else if(0 == strcmp(argv[2],"udp"))
	{
		contype=SOCK_DGRAM;
	}
	else
	{
		usage(argv[0]);
		exit(1);
	}
	print_header();
	modbus_init(&modbus);

	switch( modbus_connect(&modbus, argv[1], contype, 0) )
	{
	case 0: /* success */
		break;
	case -1: /* system error */
	addidata_network_perror("modbus_connect");
	exit(1);
	case -2: /* parameter is incorrect */
	printf("modbus_connect: incorrect parameter\n");
	exit(1);
	case -3: /* incorrect address */
	printf("incorrect address\n");
	exit(1);
	}

	assert( sample_MXCommon__GetModuleTypeEx(&modbus) == 0);
	assert( sample_MXCommon__GetTimeEx(&modbus) == 0);
	assert( sample_MXCommon__SetHardwareTriggerFilterTimeEx(&modbus, 1) == 0);

	/* sequence */
	printf("\nStarting now a sequence acquisition.\n");
	printf("Please use the monitor in the Web interface to watch the values.\n");
	printf ("Press ESC to stop the acquisition\n");
	assert( sample_MSXE360X__AnalogInputInitAndStartSequenceEx(&modbus) == 0 );
	fflush(stdout);
	addidata_console_wait_for_input();
	assert( sample_MSXE360X__AnalogInputStopAndReleaseSequenceEx(&modbus) == 0 );
	printf("\nSequence acquisition stopped.\n");

	assert( modbus_shutdown(&modbus) == 0);

	printf("\npress a key to end this sample\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	return 0;
}
